.ToastLink_toastLink__OLSeq{text-decoration:underline;font-size:var(--fidesui-font-size-md);padding-left:0;padding-right:0}.TaxonomySelect_option__vY6v2 strong{font-weight:400}.TaxonomySelect_option__vY6v2 em{visibility:hidden}.TaxonomySelect_option__vY6v2:hover strong{font-weight:600}.TaxonomySelect_option__vY6v2:hover em{visibility:visible}