(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[6355],{52923:function(e,s,n){(window.__NEXT_P=window.__NEXT_P||[]).push(["/user-management/new",function(){return n(57292)}])},57292:function(e,s,n){"use strict";n.r(s),n.d(s,{default:function(){return h}});var t=n(24246),r=n(27378),u=n(16394),i=n(86677),a=n(44121),c=n(16134),o=n(812),d=n(77830),l=n(58754),f=n(65497),w=()=>{let e=(0,i.useRouter)(),[s]=(0,f.ny)(),n=(0,c.T)();(0,r.useEffect)(()=>{n((0,f.Vv)(void 0))},[n]);let w=async n=>{let t=n.password?(0,u.JJ)(n.password):void 0,r=await s({...n,password:t});return(0,o.D4)(r)||e.push("".concat(d.e3,"/profile/").concat(r.data.id)),r};return(0,t.jsxs)("div",{children:[(0,t.jsx)(l.Z,{heading:"Users",breadcrumbItems:[{title:"All users",href:d.e3},{title:"New User"}]}),(0,t.jsx)(a.Z,{onSubmit:w})]})},_=n(77213),h=()=>(0,t.jsx)(_.Z,{title:"Users - New User",children:(0,t.jsx)(w,{})})}},function(e){e.O(0,[7476,4121,2888,9774,179],function(){return e(e.s=52923)}),_N_E=e.O()}]);