<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <title>Privacy Request Error</title>
  </head>
  <body>
    <main>
      <p>
        {{ unsent_errors }} errors have occurred when processing privacy
        requests.
      </p>
    </main>
  </body>
</html>
