(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[1855],{89772:function(e,t,r){(window.__NEXT_P=window.__NEXT_P||[]).push(["/properties/add-property",function(){return r(59575)}])},59575:function(e,t,r){"use strict";r.r(t);var n=r(24246),u=r(38355),c=r(86677),i=r(812),s=r(77213),o=r(77830),a=r(58754),d=r(46628),p=r(98795),l=r(18880),_=r(31883);t.default=()=>{let e=(0,u.pmc)(),t=(0,c.useRouter)(),[r]=(0,p.dX)(),f=async n=>{let u=await r(n);if((0,_.D4)(u)){e((0,d.Vo)((0,i.e$)(u.error)));return}let c=u.data;e((0,d.t5)("Property ".concat(n.name," created successfully"))),t.push("".concat(o.ru,"/").concat(c.id))};return(0,n.jsxs)(s.Z,{title:"Add property",children:[(0,n.jsx)(a.Z,{heading:"Properties",breadcrumbItems:[{title:"All properties",href:o.ru},{title:"Add property"}]}),(0,n.jsx)(u.xuv,{maxWidth:"720px",children:(0,n.jsx)(l.Z,{handleSubmit:f})})]})}}},function(e){e.O(0,[787,2888,9774,179],function(){return e(e.s=89772)}),_N_E=e.O()}]);