(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[3453],{35041:function(n,t,e){(window.__NEXT_P=window.__NEXT_P||[]).push(["/data-discovery/detection",function(){return e(82660)}])},82660:function(n,t,e){"use strict";e.r(t);var i=e(24246);e(27378);var c=e(35287),a=e(77830),s=e(58754),u=e(69828),r=e(1039);t.default=()=>(0,i.jsxs)(c.Z,{title:"Data detection",children:[(0,i.jsx)(s.Z,{heading:"Data detection",children:(0,i.jsx)(u.Z,{parentLink:a.XY})}),(0,i.jsx)(r.Z,{})]})}},function(n){n.O(0,[431,5309,2888,9774,179],function(){return n(n.s=35041)}),_N_E=n.O()}]);