(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[2197],{26496:function(e,i,n){(window.__NEXT_P=window.__NEXT_P||[]).push(["/404",function(){return n(77030)}])},77030:function(e,i,n){"use strict";n.r(i);var s=n(24246),t=n(72111),r=n(91067),o=n(38355),l=n(86677),c=n(52253);i.default=()=>{let e=(0,l.useRouter)();return(0,s.jsxs)("div",{children:[(0,s.jsx)(t.Z,{}),(0,s.jsx)("main",{children:(0,s.jsxs)(o.Kqy,{minH:"100vh",align:"center",justify:"center",spacing:6,children:[(0,s.jsx)(o.xuv,{bg:"white",py:16,px:[0,0,35],width:["100%","100%",640],borderRadius:4,position:["absolute","absolute","inherit"],top:0,bottom:0,left:0,right:0,boxShadow:"base",children:(0,s.jsxs)(o.Kqy,{align:"center",spacing:9,children:[(0,s.jsxs)(o.Kqy,{align:"center",justify:"center",spacing:3,children:[(0,s.jsx)(o.X6q,{fontSize:"7xl",lineHeight:"1",colorScheme:"primary",color:"gray.700",children:"Error: 404"}),(0,s.jsx)(o.xvT,{fontWeight:"semibold",children:"We’re sorry but this page doesn’t exist"}),(0,s.jsx)(o.wpx,{type:"primary",onClick:()=>e.push(c.gp),className:"w-80",children:"Return to homepage"})]}),(0,s.jsx)(o.xuv,{display:[null,null,"none"],children:(0,s.jsx)(r.Z,{src:"/logo.svg",alt:"Fides logo",width:205,height:46})})]})}),(0,s.jsx)(o.xuv,{display:["none","none","inherit"],children:(0,s.jsx)(r.Z,{src:"/logo.svg",alt:"Fides logo",width:205,height:46})})]})})]})}}},function(e){e.O(0,[2888,9774,179],function(){return e(e.s=26496)}),_N_E=e.O()}]);