(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[8161],{23321:function(e,r,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/properties/[id]",function(){return t(56041)}])},56041:function(e,r,t){"use strict";t.r(r);var n=t(24246),u=t(38355),i=t(86677),s=t(812),c=t(77213),l=t(77830),o=t(58754),p=t(46628),a=t(98795),d=t(18880),f=t(31883);r.default=()=>{let e=(0,u.pmc)(),{id:r}=(0,i.useRouter)().query,{data:t}=(0,a.J6)(r),[_]=(0,a.f2)(),h=async r=>{let{id:t,messaging_templates:n,...u}=r,i=await _({id:t,property:u});if((0,f.D4)(i)){e((0,p.Vo)((0,s.e$)(i.error)));return}e((0,p.t5)("Property ".concat(r.name," updated successfully")))};return t?(0,n.jsxs)(c.Z,{title:t.name,children:[(0,n.jsx)(o.Z,{heading:"Properties",breadcrumbItems:[{title:"All properties",href:l.ru},{title:t.name}]}),(0,n.jsx)(u.xuv,{maxWidth:"720px",children:(0,n.jsx)(d.Z,{property:t,handleSubmit:h})})]}):null}}},function(e){e.O(0,[787,2888,9774,179],function(){return e(e.s=23321)}),_N_E=e.O()}]);