"""add_callback_succeeded

Revision ID: 4b2eade4353c
Revises: d7feff990012
Create Date: 2024-05-02 19:38:46.537231

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "4b2eade4353c"
down_revision = "d7feff990012"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "requesttask", sa.Column("callback_succeeded", sa.Boolean(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("requesttask", "callback_succeeded")
