.MonitorStatusCell_errorResultTimestamp__DQQWt{color:var(--fidesui-neutral-400)}.MonitorStatusCell_errorTagText___MiU0{font-size:var(--fidesui-font-size-sm);cursor:pointer}.MonitorStatusCell_errorTagText___MiU0:hover{text-decoration:underline}.TaxonomySelect_option__vY6v2 strong{font-weight:400}.TaxonomySelect_option__vY6v2 em{visibility:hidden}.TaxonomySelect_option__vY6v2:hover strong{font-weight:600}.TaxonomySelect_option__vY6v2:hover em{visibility:visible}